@@ -289,6 +289,39 @@ fn test_dereference_link_broken_link() {
         .stderr(predicate::str::contains("No such file or directory"));
 }
 
+#[test]
+fn test_dereference_link_broken_link_output() {
+    let dir = tempdir();
+
+    let link = dir.path().join("link");
+    let target = dir.path().join("target");
+
+    #[cfg(unix)]
+    fs::symlink(target, &link).unwrap();
+
+    // this needs to be tested on Windows
+    // likely to fail because of permission issue
+    // see https://doc.rust-lang.org/std/os/windows/fs/fn.symlink_file.html
+    #[cfg(windows)]
+    std::os::windows::fs::symlink_file(&target, &link).expect("failed to create broken symlink");
+
+    cmd()
+        .arg("-l")
+        .arg("--dereference")
+        .arg("--ignore-config")
+        .arg(&link)
+        .assert()
+        .stdout(predicate::str::starts_with("l????????? ? ? ? ?"));
+
+    cmd()
+        .arg("-l")
+        .arg("-L")
+        .arg("--ignore-config")
+        .arg(link)
+        .assert()
+        .stdout(predicate::str::starts_with("l????????? ? ? ? ?"));
+}
+
 #[cfg(unix)]
 #[test]
 fn test_show_folder_content_of_symlink() {
@@ -336,7 +369,7 @@ fn test_show_folder_of_symlink_for_long_multi() {
     let dir = tempdir();
     dir.child("target").child("inside").touch().unwrap();
     let link = dir.path().join("link");
-    fs::symlink("target", &link).unwrap();
+    fs::symlink("target", link).unwrap();
 
     cmd()
         .arg("-l")
@@ -519,7 +552,7 @@ fn test_tree_no_dereference() {
     tmp.child("one.d").create_dir_all().unwrap();
     tmp.child("one.d/samplefile").touch().unwrap();
     let link = tmp.path().join("link");
-    fs::symlink("one.d", &link).unwrap();
+    fs::symlink("one.d", link).unwrap();
 
     cmd()
         .arg("--tree")
@@ -538,7 +571,7 @@ fn test_tree_dereference() {
     tmp.child("one.d").create_dir_all().unwrap();
     tmp.child("one.d/samplefile").touch().unwrap();
     let link = tmp.path().join("link");
-    fs::symlink("one.d", &link).unwrap();
+    fs::symlink("one.d", link).unwrap();
 
     cmd()
         .arg("--ignore-config")
@@ -736,3 +769,16 @@ fn test_all_directory() {
         .assert()
         .stdout(predicate::str::is_match(".").unwrap());
 }
+
+#[test]
+fn test_multiple_files() {
+    let dir = tempdir();
+    dir.child("one").touch().unwrap();
+    dir.child("two").touch().unwrap();
+
+    cmd()
+        .arg(dir.path().join("one"))
+        .arg(dir.path().join("two"))
+        .assert()
+        .stdout(predicate::str::is_match(".").unwrap());
+}
